==> tfjs-backend-wasm-3.1.0.tgz: package/src/cc/kernels/Prod.cc <==
/* Copyright 2020 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ===========================================================================*/

#ifdef __EMSCRIPTEN__
#include <emscripten.h>
#endif

#include <cstddef>

#include "src/cc/backend.h"
#include "src/cc/util.h"

namespace {

template <typename T>
void prod(const size_t x_id, const size_t reduce_size, const size_t out_id) {
  auto& x_info = tfjs::backend::get_tensor_info(x_id);
  auto& out_info = tfjs::backend::get_tensor_info_out(out_id);

  const T* x_buf = reinterpret_cast<const T *>(x_info.memory_offset);
  const size_t x_size = x_info.size;

  T* out_buf = reinterpret_cast<T *>(out_info.memory_offset);
  const size_t out_size = out_info.size;

  const T* x_offset = x_buf;

  for (size_t i = 0; i < out_size; ++i) {
    const size_t offset = i * reduce_size;
    T product = 1;

    const T* x_iter_end = x_offset + reduce_size;

    for (const T* x = x_offset; x < x_iter_end; ++x) {
      product *= (*x);
    }

    x_offset += reduce_size;
    out_buf[i] = product;
  }
}

}  // namespace

namespace tfjs {
namespace wasm {

// We use C-style API to interface with Javascript.
extern "C" {

#ifdef __EMSCRIPTEN__
EMSCRIPTEN_KEEPALIVE
#endif
void Prod(const size_t x_id, const size_t reduce_size,
          const DType dtype, const size_t out_id) {
  switch (dtype) {
    case DType::float32:
      prod<float>(x_id, reduce_size, out_id);
      break;
    case DType::int32:
      prod<int32_t>(x_id, reduce_size, out_id);
      break;
    case DType::boolean:
      prod<bool>(x_id, reduce_size, out_id);
      break;
    default:
      util::warn("Prod failed. Unknown dtype %d", dtype);
  }
}


}  // extern "C"
}  // namespace wasm
}  // namespace tfjs

==> tfjs-backend-wasm-3.2.0.tgz: package/src/cc/kernels/Prod.cc <==
/* Copyright 2020 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ===========================================================================*/

#ifdef __EMSCRIPTEN__
#include <emscripten.h>
#endif

#include <cstddef>

#include "tfjs-backend-wasm/src/cc/backend.h"
#include "tfjs-backend-wasm/src/cc/util.h"

namespace {

template <typename T>
void prod(const size_t x_id, const size_t reduce_size, const size_t out_id) {
  auto& x_info = tfjs::backend::get_tensor_info(x_id);
  auto& out_info = tfjs::backend::get_tensor_info_out(out_id);

  const T* x_buf = reinterpret_cast<const T *>(x_info.memory_offset);
  const size_t x_size = x_info.size;

  T* out_buf = reinterpret_cast<T *>(out_info.memory_offset);
  const size_t out_size = out_info.size;

  const T* x_offset = x_buf;

  for (size_t i = 0; i < out_size; ++i) {
    const size_t offset = i * reduce_size;
    T product = 1;

    const T* x_iter_end = x_offset + reduce_size;

    for (const T* x = x_offset; x < x_iter_end; ++x) {
      product *= (*x);
    }

    x_offset += reduce_size;
    out_buf[i] = product;
  }
}

}  // namespace

namespace tfjs {
namespace wasm {

// We use C-style API to interface with Javascript.
extern "C" {

#ifdef __EMSCRIPTEN__
EMSCRIPTEN_KEEPALIVE
#endif
void Prod(const size_t x_id, const size_t reduce_size,
          const DType dtype, const size_t out_id) {
  switch (dtype) {
    case DType::float32:
      prod<float>(x_id, reduce_size, out_id);
      break;
    case DType::int32:
      prod<int32_t>(x_id, reduce_size, out_id);
      break;
    case DType::boolean:
      prod<bool>(x_id, reduce_size, out_id);
      break;
    default:
      util::warn("Prod failed. Unknown dtype %d", dtype);
  }
}


}  // extern "C"
}  // namespace wasm
}  // namespace tfjs
